diff --git a/nvdaHelper/remote/ime.cpp b/nvdaHelper/remote/ime.cpp
index dc4f9f6..60094ed 100644
--- a/nvdaHelper/remote/ime.cpp
+++ b/nvdaHelper/remote/ime.cpp
@@ -305,13 +305,18 @@ static bool handleCandidates(HWND hwnd) {
 
 	/* Make sure there is at least one candidate list */
 	DWORD count = 0;
-	DWORD len = ImmGetCandidateListCountW(imc, &count);
+	ImmGetCandidateListCountW(imc, &count);
 	if (!count) {
 		ImmReleaseContext(hwnd, imc);
 		return false;
 	}
 	candidateIMEWindow=hwnd;
 	/* Read first candidate list */
+	DWORD len = ImmGetCandidateList(imc, 0, NULL, 0);
+	if(!len) {
+		ImmReleaseContext(hwnd, imc);
+		return false;
+	}
 	CANDIDATELIST* list = (CANDIDATELIST*)malloc(len);
 	ImmGetCandidateList(imc, 0, list, len);
 	ImmReleaseContext(hwnd, imc);
@@ -324,30 +329,38 @@ static bool handleCandidates(HWND hwnd) {
 	} else if (pageEnd > list->dwCount) {
 		pageEnd = list->dwCount;
 	}
-
 	/* Concatenate currently shown candidates into a string */
-	WCHAR* cand_str = (WCHAR*)malloc(len);
-	WCHAR* ptr = cand_str;
-	for (DWORD n = list->dwPageStart, count = 0;  n < pageEnd;  ++n) {
+	size_t buflen = 0;
+	for (DWORD n = list->dwPageStart;  n < pageEnd; ++n) {
 		DWORD offset = list->dwOffset[n];
 		WCHAR* cand = (WCHAR*)(((char*)list) + offset);
 		size_t clen = wcslen(cand);
-		if (!clen)  continue;
-		CopyMemory(ptr, cand, (clen + 1) * sizeof(WCHAR));
-		if ((n + 1) < pageEnd)  ptr[clen] = '\n';
-		ptr += (clen + 1);
-		++count;
+		if(clen>0) {
+			buflen += ((clen + 1) * sizeof(WCHAR));
+		}
 	}
-	HKL kbd_layout = GetKeyboardLayout(0);
-	WCHAR filename[MAX_PATH + 1]={0};
-	ImmGetIMEFileNameW(kbd_layout, filename, MAX_PATH);
-	if(cand_str&&wcslen(cand_str)>0) {
+	WCHAR* cand_str = NULL;
+	if(buflen>0) {
+		cand_str=(WCHAR*)malloc(buflen);
+		WCHAR* ptr = cand_str;
+		for (DWORD n = list->dwPageStart; n < pageEnd;  ++n) {
+			DWORD offset = list->dwOffset[n];
+			WCHAR* cand = (WCHAR*)(((char*)list) + offset);
+			size_t clen = wcslen(cand);
+			if (!clen)  continue;
+			CopyMemory(ptr, cand, (clen + 1) * sizeof(WCHAR));
+			if ((n + 1) < pageEnd)  ptr[clen] = '\n';
+			ptr += (clen + 1);
+		}
+		HKL kbd_layout = GetKeyboardLayout(0);
+		WCHAR filename[MAX_PATH + 1]={0};
+		ImmGetIMEFileNameW(kbd_layout, filename, MAX_PATH);
 		nvdaControllerInternal_inputCandidateListUpdate(cand_str,selection,filename);
+		free(cand_str);
 	}
 	/* Clean up */
-	free(cand_str);
 	free(list);
-	return (count > 0);
+	return cand_str!=NULL;
 }
 
 static WCHAR* getCompositionString(HIMC imc, DWORD index) {
